<?php

require_once('Fast.php');

class Blogcategories extends Fast {
    
    // Список указателей на категории в дереве категорий (ключ = id категории)
    private $all_blogcategories;
    // Дерево категорий
    private $blogcategories_tree;
    
    public function get_blogcategories($filter = array()) {

        if(!isset($this->blogcategories_tree)) {
            $this->init_blogcategories();
        }
        
        if(!empty($filter['product_id'])) {
            $query = $this->db->placehold("SELECT blogcategory_id FROM __products_blogcategories WHERE product_id in(?@) ORDER BY position", (array)$filter['product_id']);
            $this->db->query($query);
            $blogcategories_ids = $this->db->results('blogcategory_id');
            $result = array();
            foreach($blogcategories_ids as $id) {
                if(isset($this->all_blogcategories[$id])) {
                    $result[$id] = $this->all_blogcategories[$id];
                }
            }
            return $result;
        }
        
        $parent = null;
        if (!empty($filter['parent_id']) && isset($this->all_blogcategories[intval($filter['parent_id'])])) {
            $parent = $this->all_blogcategories[intval($filter['parent_id'])];
        }
        
        if (!empty($filter['level_depth'])) {
            $parent_filter = '';
            if ($parent) {
                $parent_filter = $this->db->placehold('AND c.id in(?@)', (array)$parent->children);
            }
            
            $this->db->query("select c.id 
                from __blogcategories c 
                where c.level_depth=? 
                $parent_filter 
                order by c.name", intval($filter['level_depth']));
            $result = array();
            foreach ($this->db->results('id') as $cid) {
                if (isset($this->all_blogcategories[$cid])) {
                    $result[$cid] = $this->all_blogcategories[$cid];
                }
            }
            return $result;
        } elseif ($parent) {
            return $parent->subblogcategories;
        }
        
        return $this->all_blogcategories;
    }
    
    public function get_product_blogcategories($product_id) {
        $query = $this->db->placehold("SELECT product_id, blogcategory_id, position FROM __products_blogcategories WHERE product_id in(?@) ORDER BY position", (array)$product_id);
        $this->db->query($query);
        return $this->db->results();
    }
    
    public function get_products_blogcategories() {
        $query = $this->db->placehold("SELECT product_id, blogcategory_id, position FROM __products_blogcategories ORDER BY position");
        $this->db->query($query);
        return $this->db->results();
    }
    
    public function get_blogcategories_tree() {
        if(!isset($this->blogcategories_tree)) {
            $this->init_blogcategories();
        }
        return $this->blogcategories_tree;
    }
    
    public function get_blogcategory($id) {
        if(!isset($this->all_blogcategories)) {
            $this->init_blogcategories();
        }
        if(is_int($id) && array_key_exists(intval($id), $this->all_blogcategories)) {
            return $blogcategory = $this->all_blogcategories[intval($id)];
        } elseif(is_string($id)) {
            foreach ($this->all_blogcategories as $blogcategory) {
                if ($blogcategory->url == $id) {
                    return $this->get_blogcategory((int)$blogcategory->id);
                }
            }
        }
        return false;
    }
    
    public function add_blogcategory($blogcategory) {
        $blogcategory = (array)$blogcategory;
        if(empty($blogcategory['url'])) {
            $blogcategory['url'] = preg_replace("/[\s]+/ui", '_', $blogcategory['name']);
            $blogcategory['url'] = strtolower(preg_replace("/[^0-9a-zа-я_]+/ui", '', $blogcategory['url']));
        }
        
        // Если есть категория с таким URL, добавляем к нему число
        while($this->get_blogcategory((string)$blogcategory['url'])) {
            if(preg_match('/(.+)_([0-9]+)$/', $blogcategory['url'], $parts)) {
                $blogcategory['url'] = $parts[1].'_'.($parts[2]+1);
            } else {
                $blogcategory['url'] = $blogcategory['url'].'_2';
            }
        }
        
        $blogcategory = (object)$blogcategory;
        if ($blogcategory->parent_id) {
            if (isset($this->all_blogcategories[$blogcategory->parent_id])) {
                $blogcategory->level_depth = $this->all_blogcategories[$blogcategory->parent_id]->level_depth + 1;
            }
        } else {
            $blogcategory->level_depth = 1;
        }
        
        $result = $this->languages->get_description($blogcategory, 'blogcategory');
        
        $blogcategory->last_modify = date("Y-m-d H:i:s");
        $this->db->query("INSERT INTO __blogcategories SET ?%", $blogcategory);
        $id = $this->db->insert_id();
        $this->db->query("UPDATE __blogcategories SET position=id WHERE id=?", $id);
        
        if(!empty($result->description)) {
            $this->languages->action_description($id, $result->description, 'blogcategory');
        }
        
        unset($this->blogcategories_tree);
        unset($this->all_blogcategories);
        return $id;
    }
    
    private function update_level_depth($id, $prev_level = 1) {
        if ($this->all_blogcategories[$id]->subblogcategories) {
            foreach ($this->all_blogcategories[$id]->subblogcategories as $sub_cat) {
                $this->db->query("UPDATE __blogcategories SET level_depth=? WHERE id=? LIMIT 1", $prev_level+1, intval($sub_cat->id));
                $this->update_level_depth($sub_cat->id, $prev_level+1);
            }
        }
    }
    
    public function update_blogcategory($id, $blogcategory) {


        if (is_object($blogcategory)) {
            if ($blogcategory->parent_id) {
                if (isset($this->all_blogcategories[$blogcategory->parent_id])) {
                    $blogcategory->level_depth = $this->all_blogcategories[$blogcategory->parent_id]->level_depth + 1;
                }
            } else {
                $blogcategory->level_depth = 1;
            }
            $this->update_level_depth($id, $blogcategory->level_depth);
        }
        
        $blogcategory = (object)$blogcategory;
        $result = $this->languages->get_description($blogcategory, 'blogcategory');
        
        $blogcategory->last_modify = date("Y-m-d H:i:s");
        $query = $this->db->placehold("UPDATE __blogcategories SET ?% WHERE id=? LIMIT 1", $blogcategory, intval($id));
        $this->db->query($query);

		
        if(!empty($result->description)) {
            $this->languages->action_description($id, $result->description, 'blogcategory', $this->languages->lang_id());
        }
        
        unset($this->blogcategories_tree);
        unset($this->all_blogcategories);
        return intval($id);
    }
    
    public function delete_blogcategory($ids) {
        $ids = (array) $ids;
        foreach($ids as $id) {
            if($blogcategory = $this->get_blogcategory(intval($id))) {
                if(!empty($blogcategory->children)) {
                    foreach ($blogcategory->children as $cid) {
                        $this->image->delete_image($cid, 'image', 'blogcategories', $this->config->original_blogcategories_dir, $this->config->resized_blogcategories_dir);
                    }
                    $query = $this->db->placehold("DELETE FROM __blogcategories WHERE id in(?@)", $blogcategory->children);
                    $this->db->query($query);
                    $query = $this->db->placehold("DELETE FROM __products_blogcategories WHERE blogcategory_id in(?@)", $blogcategory->children);
                    $this->db->query($query);
                    $this->db->query("DELETE FROM __lang_blogcategories WHERE blogcategory_id in(?@)", $blogcategory->children);
                }
            }
        }
        unset($this->blogcategories_tree);
        unset($this->all_blogcategories);
        return $id;
    }
    
    public function add_product_blogcategory($product_id, $blogcategory_id, $position=0) {
        $this->db->query("update __blogcategories set last_modify=now() where id=?", intval($blogcategory_id));
        $query = $this->db->placehold("INSERT IGNORE INTO __products_blogcategories SET product_id=?, blogcategory_id=?, position=?", $product_id, $blogcategory_id, $position);
        $this->db->query($query);
    }
    
    public function delete_product_blogcategory($product_id, $blogcategory_id) {
        $this->db->query("update __blogcategories set last_modify=now() where id=?", intval($blogcategory_id));
        $query = $this->db->placehold("DELETE FROM __products_blogcategories WHERE product_id=? AND blogcategory_id=? LIMIT 1", intval($product_id), intval($blogcategory_id));
        $this->db->query($query);
    }
    
    private function init_blogcategories() {
        // Дерево категорий
        $tree = new stdClass();
        $tree->subblogcategories = array();
        
        // Указатели на узлы дерева
        $pointers = array();
        $pointers[0] = &$tree;
        $pointers[0]->path = array();
        $pointers[0]->level = 0;
        
        $lang_sql = $this->languages->get_query(array('object'=>'blogcategory'));
        
        // Выбираем все категории
        $query = $this->db->placehold("SELECT 
                c.id, 
                c.parent_id,  
                c.url, 
                c.image, 
                c.visible, 
                c.position, 
                c.level_depth, 
                c.yandex_name, 
                c.last_modify, 
                $lang_sql->fields
            FROM __blogcategories c 
            $lang_sql->join 
            ORDER BY c.parent_id, c.position
        ");
        
        // Выбор категорий с подсчетом количества товаров для каждой. Может тормозить при большом количестве товаров.
        // $query = $this->db->placehold("SELECT c.id, c.parent_id, c.name, c.description, c.url, c.meta_title, c.meta_keywords, c.meta_description, c.image, c.visible, c.position, COUNT(p.id) as products_count
        //                               FROM __blogcategories c LEFT JOIN __products_blogcategories pc ON pc.blogcategory_id=c.id LEFT JOIN __products p ON p.id=pc.product_id AND p.visible GROUP BY c.id ORDER BY c.parent_id, c.position");
        
        $this->db->query($query);

        $blogcategories = $this->db->results();
        $finish = false;
        // Не кончаем, пока не кончатся категории, или пока ниодну из оставшихся некуда приткнуть
        while(!empty($blogcategories)  && !$finish) {
            $flag = false;
            // Проходим все выбранные категории
            foreach($blogcategories as $k=>$blogcategory) {
                if(isset($pointers[$blogcategory->parent_id])) {
                    // В дерево категорий (через указатель) добавляем текущую категорию
                    $pointers[$blogcategory->id] = $pointers[$blogcategory->parent_id]->subblogcategories[] = $blogcategory;
                    
                    // Путь к текущей категории
                    $curr = $pointers[$blogcategory->id];
                    $pointers[$blogcategory->id]->path = array_merge((array)$pointers[$blogcategory->parent_id]->path, array($curr));
                    
                    // Уровень вложенности категории
                    $pointers[$blogcategory->id]->level = 1+$pointers[$blogcategory->parent_id]->level;
                    
                    // Убираем использованную категорию из массива категорий
                    unset($blogcategories[$k]);
                    $flag = true;
                }
            }
            if(!$flag) $finish = true;
        }
        
        // Для каждой категории id всех ее деток узнаем
        $ids = array_reverse(array_keys($pointers));
        foreach($ids as $id) {
            if($id>0) {
                $pointers[$id]->children[] = $id;
                
                if(isset($pointers[$pointers[$id]->parent_id]->children)) {
                    $pointers[$pointers[$id]->parent_id]->children = array_merge($pointers[$id]->children, $pointers[$pointers[$id]->parent_id]->children);
                } else {
                    $pointers[$pointers[$id]->parent_id]->children = $pointers[$id]->children;
                }
                
                // Добавляем количество товаров к родительской категории, если текущая видима
                // if(isset($pointers[$pointers[$id]->parent_id]) && $pointers[$id]->visible)
                //		$pointers[$pointers[$id]->parent_id]->products_count += $pointers[$id]->products_count;
            }
        }
        unset($pointers[0]);
        unset($ids);
        
        $this->blogcategories_tree = $tree->subblogcategories;
        $this->all_blogcategories = $pointers;
    }

    public function get_market($query = '') {
        $query = mb_strtolower($query);
        $market_cats = array();
        $file = 'files/downloads/market_blogcategories.csv';
        if (file_exists($file)) {
            $f = fopen($file, 'r');
            fgetcsv($f, 0, '^');
            while (!feof($f)) {
                $line = fgetcsv($f, 0, '^');
                if (empty($query) || strpos(mb_strtolower($line[0]), $query) !== false) {
                    $market_cats[] = $line[0];
                }
            }
            fclose($f);
        }
        return $market_cats;
    }
    
}
