<?php

require_once('Fast.php');

class Suppliers extends Fast
{
	
    
	private $all_suppliers;
	private $suppliers_tree;

	// Функция возвращает массив поставщиков
	public function get_suppliers($filter = array())
	{
		if(!isset($this->suppliers_tree))
			$this->init_suppliers();
 
		if(!empty($filter['parent_id']))
		{
			$query = $this->db->placehold("SELECT id FROM __suppliers WHERE parent_id in(?@) ORDER BY name, position", (array)$filter['parent_id']);
			$this->db->query($query);
			$suppliers_ids = $this->db->results('id');
			$result = array();
			foreach($suppliers_ids as $id)
				if(isset($this->all_suppliers[$id]))
					$result[$id] = $this->all_suppliers[$id];
			return $result;
		}
        if(!empty($filter['product_id']))
		{
			$query = $this->db->placehold("SELECT supplier_id FROM __products_suppliers WHERE product_id in(?@) ORDER BY position", (array)$filter['product_id']);
			$this->db->query($query);
			$suppliers_ids = $this->db->results('supplier_id');
			$result = array();
			foreach($suppliers_ids as $id)
				if(isset($this->all_suppliers[$id]))
					$result[$id] = $this->all_suppliers[$id];
			return $result;
		}
		if(!empty($filter['category_id'])) {
            $category_id_filter = $this->db->placehold('LEFT JOIN __products_suppliers p ON p.supplier_id=b.id LEFT JOIN __products_categories pc ON p.product_id = pc.product_id WHERE pc.category_id in(?@)', (array)$filter['category_id']);
    		// Выбираем всех поставщиков
    		$query = $this->db->placehold("SELECT DISTINCT b.id, b.parent_id
    								 		FROM __suppliers b $category_id_filter ORDER BY b.name, b.position");
    		$this->db->query($query);
    		$suppliers_ids = $this->db->results();
			$result = array();
			foreach($suppliers_ids as $supplier) {
                $needsupplier = $supplier;
                if ($supplier->parent_id) {
                    while(isset($this->all_suppliers[$needSupplier->parent_id]) && $needSupplier->parent_id) {
                        $needSupplier = $this->all_suppliers[$needSupplier->parent_id];
                    }
                }
                if(isset($this->all_suppliers[$needSupplier->id])) {
                    $result[$needSupplier->id] = $this->all_suppliers[$needSupplier->id];
                }
			}
			return $result;
        }
		
		return $this->all_suppliers;
	}	

	// Функция возвращает id поставщиков для заданного товара
	public function get_product_suppliers($product_id)
	{
		$query = $this->db->placehold("SELECT product_id, supplier_id, position FROM __products_suppliers WHERE product_id in(?@)", (array)$product_id);
		$this->db->query($query);
		return $this->db->results();
	}	

	// Функция возвращает id поставщиков для всех товаров
	public function get_products_suppliers()
	{
		$query = $this->db->placehold("SELECT product_id, supplier_id, position FROM __products_suppliers ORDER BY name, position");
		$this->db->query($query);
		return $this->db->results();
	}	

	// Функция возвращает дерево поставщиков
	public function get_suppliers_tree()
	{
		if(!isset($this->suppliers_tree))
			$this->init_suppliers();
			
		return $this->suppliers_tree;
	}

	// Функция возвращает заданного поставщика
	public function get_supplier($id)
	{
		if(!isset($this->all_suppliers))
			$this->init_suppliers();
		if(is_int($id) && array_key_exists(intval($id), $this->all_suppliers))
			return $supplier = $this->all_suppliers[intval($id)];
		elseif(is_string($id))
			foreach ($this->all_suppliers as $supplier)
				if ($supplier->url == $id)
					return $this->get_supplier((int)$supplier->id);	
		
		return false;
	}
	
	// Добавление поставщика
	public function add_supplier($supplier)
	{
		$supplier = (array)$supplier;
		/*if(empty($supplier['url']))
		{
			$supplier['url'] = preg_replace("/[\s]+/ui", '_', $supplier['name']);
			$supplier['url'] = strtolower(preg_replace("/[^0-9a-zа-я_]+/ui", '', $supplier['url']));
		}*/	

		// Если есть поставщик с таким URL, добавляем к нему число
		while($this->get_supplier((string)$supplier['url']))
		{
			if(preg_match('/(.+)_([0-9]+)$/', $supplier['url'], $parts))
				$supplier['url'] = $parts[1].'_'.($parts[2]+1);
			else
				$supplier['url'] = $supplier['url'].'_2';
		}

		$this->db->query("INSERT INTO __suppliers SET ?%", $supplier);
		$id = $this->db->insert_id();
		$this->db->query("UPDATE __suppliers SET position=id WHERE id=?", $id);
		unset($this->suppliers_tree);	
		unset($this->all_suppliers);	
		return $id;
	}
	
	// Изменение поставщика
	public function update_supplier($id, $supplier)
	{
        $query = $this->db->placehold("UPDATE __suppliers SET ?% WHERE id=? LIMIT 1", $supplier, intval($id));
		$this->db->query($query);
		unset($this->suppliers_tree);			
		unset($this->all_suppliers);	
		return $id;
	}
	
	// Удаление поставщика
	public function delete_supplier($ids)
	{
		$ids = (array) $ids;
		foreach($ids as $id)
		{
			if($supplier = $this->get_supplier(intval($id)))
			$this->delete_image($supplier->children);
			if(!empty($supplier->children))
			{
				$query = $this->db->placehold("DELETE FROM __suppliers WHERE id in(?@)", $supplier->children);
				$this->db->query($query);
				$query = $this->db->placehold("DELETE FROM __products_suppliers WHERE supplier_id in(?@)", $supplier->children);
				$this->db->query($query);
                $query = $this->db->placehold("DELETE FROM __variants_suppliers WHERE supplier_id in(?@)", $supplier->children);
				$this->db->query($query);
			}
		}
		unset($this->suppliers_tree);			
		unset($this->all_suppliers);	
		return true;
	}
	
	// Добавить поставщика к заданному товару
	public function add_product_supplier($product_id, $supplier_id, $position=0)
	{
		$query = $this->db->placehold("INSERT IGNORE INTO __products_suppliers SET product_id=?, supplier_id=?, position=?", $product_id, $supplier_id, $position);
		$this->db->query($query);
	}

	// Удалить поставщика заданного товара
	public function delete_product_supplier($product_id, $supplier_id)
	{
		$query = $this->db->placehold("DELETE FROM __products_suppliers WHERE product_id=? AND supplier_id=? LIMIT 1", intval($product_id), intval($supplier_id));
		$this->db->query($query);
	}
	
	// Удалить изображение поставщика
	public function delete_image($suppliers_ids)
	{
		$suppliers_ids = (array) $suppliers_ids;
		$query = $this->db->placehold("SELECT image FROM __suppliers WHERE id in(?@)", $suppliers_ids);
		$this->db->query($query);
		$filenames = $this->db->results('image');
		if(!empty($filenames))
		{
			$query = $this->db->placehold("UPDATE __suppliers SET image=NULL WHERE id in(?@)", $suppliers_ids);
			$this->db->query($query);
			foreach($filenames as $filename)
			{
				$query = $this->db->placehold("SELECT count(*) as count FROM __suppliers WHERE image=?", $filename);
				$this->db->query($query);
				$count = $this->db->result('count');
				if($count == 0)
				{			
					@unlink($this->config->root_dir.$this->config->suppliers_images_dir.$filename);		
				}
			}
			unset($this->suppliers_tree);
			unset($this->all_suppliers);	
		}
	}


	// Инициализация поставщиков, после которой поставщиков будем выбирать из локальной переменной
	private function init_suppliers()
	{
		// Дерево категорий
		$tree = new stdClass();
		$tree->subsuppliers = array();
		
		// Указатели на узлы дерева
		$pointers = array();
		$pointers[0] = &$tree;
		$pointers[0]->path = array();
		
		// Выбираем все поставщиков
		$query = $this->db->placehold("SELECT c.*
										FROM __suppliers c ORDER BY c.parent_id, c.name, c.position");
											
		// Выбор поставщиков с подсчетом количества товаров для каждом. Может тормозить при большом количестве товаров.
		// $query = $this->db->placehold("SELECT c.id, c.parent_id, c.name, c.description, c.url, c.meta_title, c.meta_keywords, c.meta_description, c.image, c.visible, c.position, COUNT(p.id) as products_count
		//                               FROM __suppliers c LEFT JOIN __products_suppliers pc ON pc.supplier_id=c.id LEFT JOIN __products p ON p.id=pc.product_id AND p.visible GROUP BY c.id ORDER BY c.parent_id, c.position");
		
		
		$this->db->query($query);
		$suppliers = $this->db->results();
				
		$finish = false;
		// Не кончаем, пока не кончатся поставщики, или пока ниодну из оставшихся некуда приткнуть
		while(!empty($suppliers)  && !$finish)
		{
			$flag = false;
			// Проходим всех выбранных поставщиков
			foreach($suppliers as $k=>$supplier)
			{
				if(isset($pointers[$supplier->parent_id]))
				{
					// В дерево поставщиков (через указатель) добавляем текущего поставщика
					$pointers[$supplier->id] = $pointers[$supplier->parent_id]->subsuppliers[] = $supplier;
					
					// Путь к текущему поставщику
					$curr = $pointers[$supplier->id];
					$pointers[$supplier->id]->path = array_merge((array)$pointers[$supplier->parent_id]->path, array($curr));
					
					// Убираем использованног поставщика из массива поставщиков
					unset($suppliers[$k]);
					$flag = true;
				}
			}
			if(!$flag) $finish = true;
		}
		
		// Для каждой поставщика id всех его деток узнаем
		$ids = array_reverse(array_keys($pointers));
		foreach($ids as $id)
		{
			if($id>0)
			{
				$pointers[$id]->children[] = $id;

				if(isset($pointers[$pointers[$id]->parent_id]->children))
					$pointers[$pointers[$id]->parent_id]->children = array_merge($pointers[$id]->children, $pointers[$pointers[$id]->parent_id]->children);
				else
					$pointers[$pointers[$id]->parent_id]->children = $pointers[$id]->children;
					
				// Добавляем количество товаров к родительскому поставщику, если текущый видимый
				// if(isset($pointers[$pointers[$id]->parent_id]) && $pointers[$id]->visible)
				//		$pointers[$pointers[$id]->parent_id]->products_count += $pointers[$id]->products_count;
			}
		}
		unset($pointers[0]);
		unset($ids);

		$this->suppliers_tree = $tree->subsuppliers;
		$this->all_suppliers = $pointers;	
	}
    
    public function get_variant_supplier_info($filter=array()){
        $variant_id_filter = '';
        $supplier_id_filter = '';
        if(isset($filter['variant_id']))
            $variant_id_filter = $this->db->placehold(" AND variant_id=?",intval($filter['variant_id']));
        if(isset($filter['supplier_id']))
            $supplier_id_filter = $this->db->placehold(" AND supplier_id=?",intval($filter['supplier_id']));
        $this->db->query("SELECT * FROM __variants_suppliers WHERE 1 $variant_id_filter $supplier_id_filter");
        
        return $this->db->results();
    }
    public function add_variant_supplier_info($info=array()){
        if(empty($info))
            return false;
        $this->db->query("INSERT INTO __variants_suppliers SET ?%", $info);
    }
    public function delete_variant_supplier($variant_id){
        if(!is_int($variant_id))
            return false;
        $query = $this->db->placehold("DELETE FROM __variants_suppliers WHERE variant_id=?", intval($variant_id));
		$this->db->query($query);
    }

}