<!DOCTYPE html>
<html {if $language->label}lang="{$language->label|escape}"{/if} prefix="og: http://ogp.me/ns#">
<head>
	{* Полный базовый адрес *}
	<base href="{$config->root_url}/"/>

	{* Метатеги *}
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>

	{* Тайтл страницы *}
    <title>{$meta_title|escape}{$filter_meta->title|escape}</title>
    {if (!empty($meta_description) || !empty($meta_keywords) || !empty($filter_meta->description) || !empty($filter_meta->keywords)) && !$smarty.get.page}
        <meta name="keywords" content="{$meta_keywords|escape}{$filter_meta->keywords|escape}"/>
        <meta name="description" content="{$meta_description|escape}{$filter_meta->description|escape}"/>
    {/if}
	
	{* Контроль индексации *}
	{if $module == 'ProductsView' || $smarty.get.module == 'ProductView' || $smarty.get.module == 'MainView' || $smarty.get.module == "BlogView" || $smarty.get.module == "PageView"}
		<meta name="robots" content="index,follow"/>
	{elseif $smarty.get.module == 'BrandsView' || $smarty.get.module == "BlogcategoryView" || ($smarty.get.module == "BlogView" && !$smarty.get.url) || $smarty.get.page || isset($smarty.get.keyword)}
		<meta name="robots" content="noindex,follow"/>
	{else}
		<meta name="robots" content="noindex,nofollow"/>
	{/if}
	<meta name="viewport" content="width=device-width, user-scalable=yes, initial-scale=1.0"/>
	<meta name="generator" content="FastCMS {$config->version}" />

    {if $settings->g_webmaster}
        <meta name="google-site-verification" content="{$settings->g_webmaster}" />
    {/if}
    {if $settings->y_webmaster}
        <meta name='yandex-verification' content="{$settings->y_webmaster}" />
    {/if}

	{$rel_prev_next}

	{* Изображения товара и поста для соц. сетей *}
    {if $module == 'ProductView'}
        <meta property="og:url" content="{$config->root_url}{if $lang_link}/{str_replace('/', '', $lang_link)}{/if}{$canonical}"/>
        <meta property="og:type" content="product"/>
        <meta property="og:title" content="{$product->name|escape}"/>
        <meta property="og:description" content="{$product->body|escape|truncate:295:'...'}"/>
		<meta property="og:image:width" content="300"/>
		<meta property="og:image:height" content="300"/>
        <meta property="og:image" content="{$product->image->filename|resize:300:300}"/>

	{elseif $module == 'BlogView'}
		<meta property="og:url" content="{$config->root_url}{if $lang_link}/{str_replace('/', '', $lang_link)}{/if}{$canonical}"/>
		<meta property="og:type" content="article"/>
		<meta property="og:title" content="{$post->name|escape}"/>
		<meta property="og:description" content="{if $post->annotation}{$post->annotation|strip_tags|truncate:295:'...'}{else}{$post->text|strip_tags|truncate:295:'...'}{/if}"/>
		{if $post->image}
			<meta property="og:image:width" content="300"/>
			<meta property="og:image:height" content="300"/>
			<meta property="og:image" content="{$post->image|resize:300:300:false:$config->resized_blog_dir}"/>
		{else}
			<meta property="og:image" content="{$config->root_url}/design/{$settings->theme}/images/logo{if $language->label}_{$language->label}{/if}.png"/>
		{/if}
	{else}
		<meta property="og:title" content="{$settings->site_name}" />
		<meta property="og:type" content="website"/>
		<meta property="og:url" content="{$config->root_url}"/>
		<meta property="og:image" content="{$config->root_url}/design/{$settings->theme}/images/logo{if $language->label}_{$language->label}{/if}.png" />
		<meta property="og:site_name" content="{$settings->site_name}"/>
		<meta property="og:description" content="{$meta_description|escape}"/>
	{/if}

	{* Канонический адрес страницы *}
	{if isset($canonical)}
        <link rel="canonical" href="{$config->root_url}{if $lang_link}/{str_replace('/', '', $lang_link)}{/if}{$canonical}"/>
    {/if}

	{* Языковый атрибут *}
    {foreach $languages as $l}
		{if $l->enabled}
            <link rel="alternate" hreflang="{$l->label}" href="{$config->root_url}/{$l->url}"/>
		{/if}
	{/foreach}

	{* Иконка сайта *}
	<link rel="icon" type="image/png" href="design/{$settings->theme|escape}/images/favicon.png">
	<meta name="theme-color" content="#ffffff">
	
	{* JQuery *}
	<script src="https://ajax.googleapis.com/ajax/libs/jquery/2.2.4/jquery.min.js"></script>

	{* CSS *}
	<link href="/design/{$settings->theme|escape}/css/style.css" rel="stylesheet"/>
	{if $is_mobile || $is_tablet}
		<link href="/design/{$settings->theme|escape}/css/mobile_menu.css" rel="stylesheet"/>
	{/if}

	{* Скрипты *}
	{include file="scripts.tpl"}
	
</head>
<body class="page fullWidth {$module}">
<div>
<noscript><div class="noscript">У вас отключены Java скрипты, включите их для корректной работы магазина</div></noscript>																	 

{* Шапка *}
<header>

	<div id="header">
		
		<div class="container topmenu">
			<div id="main" class="flex">
				{if $is_mobile}
					<div class="logo hidden-sm-up center">
						<a href="/{$lang_link}">
							<img class="img-fluid logo_fix" src="design/{$settings->theme|escape}/images/logo{if $language->label}_{$language->label}{/if}.png" alt="{$settings->site_name|escape}"/>
						</a>
					</div>
				{/if}
					
				<div id="top" class="pages_menu overlay-menu">
					<div class="overlay" id="overlay">
						<nav>
							<ul class="list-inline">
							{foreach $pages as $p}
							{if $p->menu_id == 1 && $p->url}
							<li><a data-page="{$p->id}" href="/{$lang_link}{$p->url}">{$p->name|escape}</a></li>
							{/if}
							{/foreach}
							</ul>
						</nav>
					</div>
				</div>

				<div class="text-plus-icon info">
					<a class="name nowrap" href="tel:{$lang->index_phone_1|replace:' ':''}" data-language="{$translate_id['index_phone_1']}"><i class="ico-head_phone1"></i>{$lang->index_phone_1}</a>
					{if $lang->index_phone_2}
					<a class="name nowrap" href="tel:{$lang->index_phone_2|replace:' ':''}" data-language="{$translate_id['index_phone_2']}"><i class="ico-head_phone1"></i>{$lang->index_phone_2}</a>
					{/if}
				</div>
				
				<div class="small" data-language="{$translate_id['working_hours']}">{$lang->working_hours}</div>
				{* Обратный звонок *}
				
				<a class="pro_callback btn btn-primary" href="#pro_callback" data-language="{$translate_id['index_back_call']}">{$lang->index_back_call}</a>
					
			</div>
		</div>
		<div id='catMenu'>
			{if $is_mobile || $is_tablet}
				{include "mmenu.tpl"}
			{/if}
			
			{include "menu.tpl"}
			
		</div>
	</div>
</header>

				
	
<div class="wrapper">

			{* Поиск для мобильников *}
			{if $is_mobile}			
				<div class="mobile search">
					<form id="pro_search" action="{$lang_link}all-products">
						<input class="pro_search " type="text" name="keyword" value="{$keyword|escape}" data-language="{$translate_id['index_search']}" placeholder="{$lang->index_search}"/>
						<button class="ico-search" type="submit"></button>
					</form>
				</div>
			{/if}

	{* Баннер *}
	{if $smarty.get.module == "MainView"}
	{get_banner var=banner_group1 group='group1'}
		{if $banner_group1->items}	
		<div class="banner slick-banner" id="slider">
			{foreach $banner_group1->items as $bi}
				<div class="slick-img" style="background:url({$config->banners_images_dir}{$bi->image})">
					<div class="overlay-dots">
					
						{if $bi->url}
						<a href="{$bi->url}" target="_blank">
						{/if}
						
						<div class="preloader">
							<div class="status"></div>
						</div>
						<div class="container">
								<div class="slick-name">
									{$bi->title}
								</div>
								{if $bi->description}
								<div class="slick-description">
								{$bi->description}
								</div>
								{/if}
								{if $bi->url}
									<div class='btn btn-success'>{if $bi->alt}{$bi->alt}{else}Подробнее{/if}</div>
								{/if}
						</div>
					
						{if $bi->url}
						</a>
						{/if}
						
					</div>
				</div>
			{/foreach}
		</div>
		{/if}
	{/if}

	{* info *}
	{if $smarty.get.module == "MainView"}
	<div id="info-bar" class="center hidden-sm-down">
		<div class="container p-y-1">
		<div class="row">
			<div class="col-xs-12 col-md-6 col-lg-4 p-a-1">
				<div class="text-plus-icon f-truck" data-language="{$translate_id['index_block1']}">{$lang->index_block1}</div>
			</div>
			<div class="col-xs-12 col-md-6 col-lg-4 p-a-1">
				<div class="text-plus-icon f-map" data-language="{$translate_id['index_block2']}">{$lang->index_block2}</div>
			</div>
			<div class="col-xs-12 hidden-md-down col-lg-4 p-a-1">
				<div class="text-plus-icon ico-pr_hit" data-language="{$translate_id['index_block3']}">{$lang->index_block3}</div>
			</div>
		</div>
		</div>
	</div>
	{/if}

	<div class="container">

		{if $smarty.get.module != "MainView"}
		<div class="breadcrumb" id=content_link>
			{include file='breadcrumb.tpl'}
		</div>
		{/if}

		{* Наполнение страницы, в зависимости от модуля {$smarty.get.module}*}
		<div class="content-block">

			{if $smarty.get.module != "ProductsView"}
				<div id="pro_content">{$content}</div>

			{else}
			<div>
				<div class="row">
					<aside>
						<div class="content-aside col-xs-12 m-b-1">
							{if $smarty.get.module == "ProductsView" && $products}
								<div class="fn_features_wrap">{include "features.tpl"}</div>
							{elseif $smarty.get.module == "ProductsView" && !$products}
								<div class="center p-t-1"><div class="center btn btn-sm" onclick="location.href='{$smarty.server.REQUEST_URI}?cancelfilter=1';">Cбросить фильтр</div></div>
							{/if}
							{if $last_posts}
								{include "aside.tpl"}
							{/if}
						</div>
					</aside>
						<div class="page_content col-xs-12" id="pro_content">
							{$content}
						</div>
				</div>
			</div>
			{/if}
		</div>

		{if $smarty.get.module != "OrderView" && $smarty.get.module != "FeedbackView"}
			{* Просмотренные товары *}
			{get_browsed_products var=browsed_products limit=12}
			{if $browsed_products}
			<div class="p-y-1 hidden-sm-down">
				<div>
					{* Заголовок блока *}
					<div class="heading dotted">
						<div class="h1 title" data-language="{$translate_id['features_browsed']}"><strong>{$lang->features_browsed}</strong></div>
						<div class="subText">Вы можете отложить товары <a class='text-underline' href='wishlist'>в закладки</a>, где всегда найдете их.</div>
					</div>
					<div class="browsed_products slick-products">
					{foreach $browsed_products as $browsed_product}
						<div class="col-md-4 col-xl-3 center">
							<a href="/{$lang_link}products/{$browsed_product->url}">
								<div class="image">
								{if $browsed_product->image->filename}
									<img src="{$browsed_product->image->filename|resize:180:180}" alt="Купить {$browsed_product->name|escape}" title="{$browsed_product->name|escape}">
								{else}
									<img width="180" height="180" src="design/{$settings->theme}/images/noimage.jpg" alt="Купить {$browsed_product->name|escape}" title="{$browsed_product->name|escape}"/>
								{/if}
								</div>
								<div class="name">{$browsed_product->name|escape}</div>
							</a>
						</div>
					{/foreach}
					</div>
				</div>
			</div>
			
			
			
			{/if}
		{/if}
		

	</div>
</div>

{* Подвал сайта *}
<footer>
	<div id="footer" class="p-t-1">
		<div class="container p-a-0">
			<div class="col-xs-12 col-lg-3">
				<div class="h4 text-caps text-bold left" data-language="{$translate_id['footer_info_header3']}">{$lang->footer_info_header3}</div>
				<div>
					<a class="f-phone_3 d-block" href="tel:{$lang->index_phone_1|replace:' ':''}" data-language="{$translate_id['index_phone_1']}" >{$lang->index_phone_1}</a>
					{if $lang->index_phone_2}
					<a class="f-phone_3 d-block" href="tel:{$lang->index_phone_2|replace:' ':''}" data-language="{$translate_id['index_phone_2']}" >{$lang->index_phone_2}</a>
					{/if}
					
					{if $lang->working_hours}
					<div class="f-clock" data-language="{$translate_id['working_hours']}">{$lang->working_hours}</div>
					{/if}
					{if $lang->footer_map}
					<div class="f-map" data-language="{$translate_id['footer_map']}">{$lang->footer_map}</div>
					{/if}
					{if $lang->footer_mail}
					<div class="f-mail" data-language="{$translate_id['footer_mail']}">{$lang->footer_mail}</div>
					{/if}
					{* Обратный звонок *}
					<a class="pro_callback" href="#pro_callback" data-language="{$translate_id['index_back_call']}">{$lang->index_back_call}</a>
				</div>
				
				<div data-language="{$translate_id['footer_info']}">{$lang->footer_info}</div>
			</div>
			
			<div class="col-xs-12 col-sm-12 col-lg-9 col-xl-6">
				
				<div class="h4 text-caps text-bold left" data-language="{$translate_id['footer_info_header1']}">{$lang->footer_info_header1}</div>
				<ul class="list-inline nav-link-line left m-b-1">
					{foreach $pages as $p}
						{if $p->menu_id == 1}
						<li><a href="/{$p->url}" data-page="{$p->id}">{$p->name|escape}</a></li>
						{/if}
					{/foreach}
				</ul>
				
				{*
				<div class="h4 text-caps text-bold left" data-language="{$translate_id['footer_info_header2']}">{$lang->footer_info_header2}</div>
				<div class="nav-link-line left column-count3">
					{assign var="count" value=0}
					{function name=categories_footer}
					{if $categories}
						{foreach $categories as $c}
						{if $c->visible && $count <40}
							{$count = $count + 1}
							<div class="display-inline"><a href="catalog/{$c->url}" data-category="{$c->id}">{$c->name|escape}</a></div>
						{/if}
						{/foreach}
					{/if}
					{/function}
					{categories_footer categories=$categories}
				</div>
				*}
			</div>
			
			<div class="col-xs-12 col-xl-3 center p-r-0 p-a-0-sm pull-lg-right">
				<div class="col-xs-12 col-md-6 col-xl-12 p-b-1 p-t-0 center">
					<div class="h4 text-caps text-bold">Поделитесь ссылкой</div>
					<div class="ya-share2" data-services="vkontakte,viber,whatsapp,telegram"></div>
				</div>
				
				{* Подписка на рассылку *}
				<div class="col-xs-12 col-md-6 col-xl-12 p-b-1 p-t-0 center">
					<div class="h4 text-caps" data-language="{$translate_id['index_subscribe']}">{$lang->index_subscribe}</div>
					<form class="subscribe-form" method="post">
						<input type="hidden" name="subscribe" value="1"/>
						<input class="form-control subscribe-control" type="email" name="subscribe_email" value="" data-format="email" data-notice="{$lang->form_enter_email}" placeholder="{$lang->form_email}"/>
						<input class="btn btn-primary btn-subscribe" type="submit" value="OK"/>

						{if $subscribe_error}
						<div id="subscribe_error" class="error">
							{if $subscribe_error == 'email_exist'}<div data-language="{$translate_id['index_subscribe_sent']}">{$lang->index_subscribe_already}</div>{/if}
							{if $subscribe_error == 'empty_email'}<div data-language="{$translate_id['form_enter_email']}">{$lang->form_enter_email}</div>{/if}
						</div>
						{/if}
						{if $subscribe_success}
						<div class="hidden">
							<div id="pro_subscribe-sent" class="p-a-1">
								<div class="h1" data-language="{$translate_id['index_subscribe_sent']}">
									<div class="col-xs-12 text-center">
									<a href="/{$lang_link}"><img class="center-block" src="design/{$settings->theme|escape}/images/logo{if $language->label}_{$language->label}{/if}.png" alt="{$settings->site_name|escape}"/></a>
									</div>
									<div class="text-center">
									{$lang->index_subscribe_sent}
									</div>
								</div>
							</div>
						</div>
						{/if}
					</form>
				</div>
				
				{* Скрываем ссылку от поисковиков, выводим только для браузеров. Просьба не удалять, это поможет другим начинающим предпринимателям найти готовое решение без мучений с доработками. *}
				{if !empty($smarty.cookies)}
					<a href="https://fastcms.ru" target="_blank">Работает на FastCMS</a>
				{/if}
				
			</div>
		</div>
		
	</div>
</footer>

<a href="#" class="inpage-scroll scrollUp"><i class="f-up"></i><span>наверх</span></a>

{* Форма обратного звонка *}
{include file='callback.tpl'}

{* Popup при клике по корзине*}
{include file='cart_popup.tpl'}

{if $settings->y_metric}
    {literal}
    
	<!-- Yandex.Metrika counter -->
	<script type="text/javascript" >
	   (function(m,e,t,r,i,k,a){m[i]=m[i]||function(){(m[i].a=m[i].a||[]).push(arguments)};
	   m[i].l=1*new Date();k=e.createElement(t),a=e.getElementsByTagName(t)[0],k.async=1,k.src=r,a.parentNode.insertBefore(k,a)})
	   (window, document, "script", "https://mc.yandex.ru/metrika/tag.js", "ym");

	   ym({/literal}{$settings->y_metric}{literal}, "init", {
			clickmap:true,
			trackLinks:true,
			accurateTrackBounce:true,
			webvisor:true,
			ecommerce:"dataLayer"
	   });
	</script>
	<noscript><div><img src="https://mc.yandex.ru/watch/{/literal}{$settings->y_metric}{literal}" style="position:absolute; left:-9999px;" alt="" /></div></noscript>
	<!-- /Yandex.Metrika counter -->
	
{/literal}
{/if}

{if $settings->g_analytics}
{literal}
	<!-- Global site tag (gtag.js) - Google Analytics -->
	<script async src="https://www.googletagmanager.com/gtag/js?id={/literal}'{$settings->g_analytics}'{literal}"></script>
	<script>
	  window.dataLayer = window.dataLayer || [];
	  function gtag(){dataLayer.push(arguments);}
	  gtag('js', new Date());

	  gtag('config', {/literal}'{$settings->g_analytics}'{literal});
	</script>

{/literal}
{/if}

{*заказ в 1 клик*}
{if $smarty.get.module == "MainView" || $smarty.get.module == "ProductsView" || $smarty.get.module == "ProductView" || $smarty.get.module == "ComparisonView" || $smarty.get.module == "WishlistView"}
    {include file='fast_order.tpl'}
{/if}
{*/заказ в 1 клик*}	

{* Остальные скрипты *}
<script src="design/{$settings->theme}/js/js.js"></script>
{if $is_mobile || $is_tablet}
	<script src="design/{$settings->theme}/js/jquery.mmenu.js"></script>
	<script type="text/javascript">
		$("nav#menu").mmenu({
		  navbar: {
			title: "Каталог"
		  }
		});
	</script>
{/if}

{* поделиться в соц. сетях / API Yandex *}
<script src="//yastatic.net/es5-shims/0.0.2/es5-shims.min.js" async="async"></script>
<script src="//yastatic.net/share2/share.js" async="async"></script>

{* Всплывающие подсказки для администратора *}
{if $smarty.session.admin}
<script src ="admin/design/js/admintooltip/admintooltip.js" async="async"></script>
<link href="admin/design/js/admintooltip/styles/admin.css" rel="stylesheet"/>
{/if}

{* Блокировка правого клика
{if !$smarty.session.admin}
	{literal}
	<script>
	$(document).ready(function(){$(document).bind("contextmenu",function(e){return false;});});
	</script>
	{/literal}
{/if}
*}
</div>
</body>
</html>