<?php

require_once('View.php');

class ProductView extends View {
    
    public function fetch() {
        $product_url = $this->request->get('product_url', 'string');
        
        if(empty($product_url)) {
            return false;
        }
        
        // Выбираем товар из базы
        $product = $this->products->get_product((string)$product_url);
        if(empty($product) || (!$product->visible && empty($_SESSION['admin']))) {
            return false;
        }
        
        //lastModify
        $this->setHeaderLastModify($product->last_modify);
        
        $product->images = $this->products->get_images(array('product_id'=>$product->id));
        $product->image = reset($product->images);
        /*изображение варианта*/
        $temp_images = array();
        /*/изображение варианта*/		   
        foreach ($product->images as $image) {
            if(substr($image->filename, 0, 7) == 'http://') {
                if($filename = $this->image->download_image($image->filename)) {
                    $image->filename = $filename;
                }
            }
            /*изображение варианта*/
            $temp_images[$image->id] = $image;
            /*/изображение варианта*/						   
        }
        
        $variants = array();
        foreach($this->variants->get_variants(array('product_id'=>$product->id)) as $v) {
            /*изображение варианта*/
            $variant_images = array();
            foreach($v->images as $iid) {
                $variant_images[] = $temp_images[$iid];
            }
            if (!empty($variant_images)) {
                $v->images_ids = $v->images;
                $v->images = $variant_images;
            } else {
                $v->images_ids = array_keys($temp_images);
                $v->images = $product->images;
            }
            $v->image = reset($v->images);
            /*/изображение варианта*/			   
            $variants[$v->id] = $v;
        }
        
        $product->variants = $variants;
        
        // Вариант по умолчанию
        if(($v_id = $this->request->get('variant', 'integer'))>0 && isset($variants[$v_id])) {
            $product->variant = $variants[$v_id];
        } else {
            $product->variant = reset($variants);
        }
        $product->features = $this->features->get_product_options(array('product_id'=>$product->id, 'in_product'=>1));
        /*мультихарактеристики*/
        $temp_options = array();
        foreach($product->features as $option) {
            if (!isset($temp_options[$option->feature_id])) {
                $temp_options[$option->feature_id] = new stdClass();
            }
            $temp_options[$option->feature_id]->feature_id = $option->feature_id;
            $temp_options[$option->feature_id]->auto_name_id = $option->auto_name_id;
            $temp_options[$option->feature_id]->auto_value_id = $option->auto_value_id;
            $temp_options[$option->feature_id]->is_color = $option->is_color;
            $temp_options[$option->feature_id]->name = $option->name;
            $temp_options[$option->feature_id]->values[] = $option->value;
        }
        foreach($temp_options as $id => $option) {
            $temp_options[$id]->value = implode(', ', $temp_options[$id]->values);
        }
        $product->features = $temp_options;
        /*/мультихарактеристики*/	
        
        // Автозаполнение имени для формы отзыва
        if(!empty($this->user)) {
            $this->design->assign('comment_name', $this->user->name);
            $this->design->assign('comment_email', $this->user->email);
        }
        
        // Принимаем отзыв
        if ($this->request->method('post') && $this->request->post('comment')) {
            $comment = new stdClass;
            $comment->name = $this->request->post('name');
            $comment->email = $this->request->post('email');
            $comment->text = $this->request->post('text');
            $captcha_code =  $this->request->post('captcha_code', 'string');
            
            // Передадим отзыв обратно в шаблон - при ошибке нужно будет заполнить форму
            $this->design->assign('comment_name', $comment->name);
			$this->design->assign('lastname',  $comment->lastname);
            $this->design->assign('comment_email', $comment->email);
            $this->design->assign('comment_text', $comment->text);
            
            // Проверяем капчу и заполнение формы
			if (!$this->validate->is_name($comment->name, true)) {
                $this->design->assign('error', 'empty_name');
			}
			elseif(!$this->validate->is_lastname($comment->lastname, true)) {
                $this->design->assign('error', 'empty_name');
            }
			elseif(!$this->validate->is_email($comment->email, true)) {
                $this->design->assign('error', 'empty_email');
            }
			elseif (!$this->validate->is_comment($comment->text, true)) {
                $this->design->assign('error', 'empty_comment');
            }
            elseif ($this->settings->captcha_product && ($_SESSION['captcha_code'] != $captcha_code || empty($captcha_code))) {
                $this->design->assign('error', 'captcha');
            }
			else {
                // Создаем отзыв
                $comment->object_id = $product->id;
                $comment->type      = 'product';
                $comment->ip        = $_SERVER['REMOTE_ADDR'];
                $comment->lang_id   = $_SESSION['lang_id'];
                
                // Если были одобренные отзывы от текущего ip, одобряем сразу
                $this->db->query("SELECT 1 FROM __comments WHERE approved=1 AND ip=? LIMIT 1", $comment->ip);
                if($this->db->num_rows()>0) {
                    $comment->approved = 1;
                }
                
                // Добавляем отзыв в базу
                $comment_id = $this->comments->add_comment($comment);
                
                // Отправляем email
                $this->notify->email_comment_admin($comment_id);
                
                // Приберем сохраненную капчу, иначе можно отключить загрузку рисунков и постить старую
                unset($_SESSION['captcha_code']);
                header('location: '.$_SERVER['REQUEST_URI'].'#comment_'.$comment_id);
            }
        }
        
        // Связанные товары
        $related_ids = array();
        $related_products = array();
        foreach($this->products->get_related_products($product->id) as $p) {
            $related_ids[] = $p->related_id;
            $related_products[$p->related_id] = null;
        }
        if(!empty($related_ids)) {
            foreach($this->products->get_products(array('id'=>$related_ids,'limit' => count($related_ids),'visible'=>1, 'in_stock'=>1)) as $p) {
                $related_products[$p->id] = $p;
            }
            /*изображение варианта*/
            $temp_images = array();
            /*/изображение варианта*/					   
            $related_products_images = $this->products->get_images(array('product_id'=>array_keys($related_products)));
            foreach($related_products_images as $related_product_image) {
                if(isset($related_products[$related_product_image->product_id])) {
                    $related_products[$related_product_image->product_id]->images[] = $related_product_image;
                }
                /*изображение варианта*/
                $temp_images[$related_product_image->id] = $related_product_image;
                /*/изображение варианта*/					   
            }
            $related_products_variants = $this->variants->get_variants(array('product_id'=>array_keys($related_products)));
            foreach($related_products_variants as $related_product_variant) {
                if(isset($related_products[$related_product_variant->product_id])) {
					/*изображение варианта*/
                    $variant_images = array();
                    foreach($related_product_variant->images as $iid) {
                        $variant_images[] = $temp_images[$iid];
                    }
                    $related_product_variant->images = !empty($variant_images) ? $variant_images : $related_products[$related_product_variant->product_id]->images;
                    $related_product_variant->image = isset($related_product_variant->images) ? reset($related_product_variant->images): null;
                    /*/изображение варианта*/					   
                    $related_products[$related_product_variant->product_id]->variants[] = $related_product_variant;
                }
            }
            foreach($related_products as $id=>$r) {
                if(is_object($r)) {
                    $r->image = $r->images[0];
                    $r->variant = $r->variants[0];
                } else {
                    unset($related_products[$id]);
                }
            }
            $this->design->assign('related_products', $related_products);
        }

        //Связянные статьи для товара
        $related_post = array();
        $related_post = $this->blog->get_related_products(array('product_id'=>$product->id));
        if(!empty($related_post)) {
            $filter_post['visible'] = 1;
            foreach ($related_post as $r_post) {
                $filter_post['id'][] = $r_post->post_id;
            }
            $posts = $this->blog->get_posts($filter_post);
            $this->design->assign('related_posts', $posts);
        }
        
        // Отзывы о товаре
        $comments = $this->comments->get_comments(array('has_parent'=>false, 'type'=>'product', 'object_id'=>$product->id, 'approved'=>1, 'ip'=>$_SERVER['REMOTE_ADDR']));
        $children = array();
        foreach ($this->comments->get_comments(array('has_parent'=>true, 'type'=>'product', 'object_id'=>$product->id, 'approved'=>1, 'ip'=>$_SERVER['REMOTE_ADDR'])) as $c) {
            $children[$c->parent_id][] = $c;
        }

        // И передаем его в шаблон
        $this->design->assign('product', $product);
        $this->design->assign('comments', $comments);
        $this->design->assign('children', $children);
        
        // Категория и бренд товара
        $product->categories = $this->categories->get_categories(array('product_id'=>$product->id));
        $this->design->assign('brand', $this->brands->get_brand(intval($product->brand_id)));
        $category = reset($product->categories);
        $this->design->assign('category', $category);

        // Соседние товары
        if (!empty($category)) {
            $neighbors_products = $this->products->get_neighbors_products($category->id, $product->position);
            $this->design->assign('next_product', $neighbors_products['next']);
            $this->design->assign('prev_product', $neighbors_products['prev']);
        }
        
        // Добавление в историю просмотров товаров
        $max_visited_products = 100; // Максимальное число хранимых товаров в истории
        $expire = time()+60*60*24*30; // Время жизни - 30 дней
        if(!empty($_COOKIE['browsed_products'])) {
            $browsed_products = explode(',', $_COOKIE['browsed_products']);
            // Удалим текущий товар, если он был
            if(($exists = array_search($product->id, $browsed_products)) !== false) {
                unset($browsed_products[$exists]);
            }
        }
        // Добавим текущий товар
        $browsed_products[] = $product->id;
        $cookie_val = implode(',', array_slice($browsed_products, -$max_visited_products, $max_visited_products));
        setcookie("browsed_products", $cookie_val, $expire, "/");
        
		// Если у товара указаны все метатеги то упрощаем конструкцию
		if (!empty($product->meta_title) and !empty($product->meta_title) and !empty($product->meta_title)) {
			$this->design->assign('meta_title', $product->meta_title);
			$this->design->assign('meta_keywords', $product->meta_keywords);
			$this->design->assign('meta_description', $product->meta_description);
		//Иначе берем метатеги и описание из шаблона
		} else {
            $parts = array(
                '{категория}' => ($category->name ? $category->name : ''),
                '{категория_h1}' => ($category->name_h1 ? $category->name_h1 : ''),
                '{бренд}' => ($this->design->get_var('brand') ? $this->design->get_var('brand')->name : ''),
                '{название}' => ($product->name ? $product->name : ''),
				'{артикул}' => ($product->variant->sku ? $product->variant->sku : ''),
                '{цена}' => ($product->variant->price != null ? $this->money->convert($product->variant->price, $this->currency->id, false).' '.$this->currency->sign : ''),
                '{магазин}' => ($this->settings->site_name ? $this->settings->site_name : '')
            );
            foreach ($product->features as $feature) {
                if ($feature->auto_name_id) {
                    $parts['{$'.$feature->auto_name_id.'}'] = $feature->name;
                }
                if ($feature->auto_value_id) {
                    $parts['{$'.$feature->auto_value_id.'}'] = $feature->value;
                }
            }
			
            $auto_meta_title = $category->auto_meta_title;
            $auto_meta_keywords = $category->auto_meta_keywords;
            $auto_meta_description = $category->auto_meta_desc;
			
			$all_p_meta_title = $this->settings->all_p_meta_title;
			$all_p_meta_keywords = $this->settings->all_p_meta_keywords;
			$all_p_meta_description = $this->settings->all_p_meta_description;

            $auto_meta_title = strtr($auto_meta_title, $parts);
            $auto_meta_keywords = strtr($auto_meta_keywords, $parts);
            $auto_meta_description = strtr($auto_meta_description, $parts);
			
			$all_p_meta_title = strtr($all_p_meta_title, $parts);
			$all_p_meta_keywords = strtr($all_p_meta_keywords, $parts);
			$all_p_meta_description = strtr($all_p_meta_description, $parts);
			
            if (empty($product->body)) {
				if (!empty($category->auto_body)){
                $product->body = strtr($category->auto_body, $parts);
                $product->body = preg_replace('/\{\$[^\$]*\}/', '', $product->body);
				}else{
				$product->body = strtr($this->settings->all_p_body, $parts);
                $product->body = preg_replace('/\{\$[^\$]*\}/', '', $product->body);
				}
            }
            $auto_meta_title = preg_replace('/\{\$[^\$]*\}/', '', $auto_meta_title);
            $auto_meta_keywords = preg_replace('/\{\$[^\$]*\}/', '', $auto_meta_keywords);
            $auto_meta_description = preg_replace('/\{\$[^\$]*\}/', '', $auto_meta_description);
			
			//Уберем знаки препинания в ключевых словах
			$bad = array(",",".","/",":",";","%","!","?","-","+",'"',"'","  ","   ");
			$auto_meta_keywords = str_replace($bad, " ", $auto_meta_keywords);
			$all_p_meta_keywords = str_replace($bad, " ", $all_p_meta_keywords);
			
			// берем метаданные у товара, если их нет - то из шаблона категории, иначе из общего шаблона
			if (!empty($product->meta_title)){
            $this->design->assign('meta_title', $product->meta_title);
			}elseif(!empty($category->auto_meta_title)){
				$this->design->assign('meta_title', $auto_meta_title);
			}else{ $this->design->assign('meta_title', $all_p_meta_title);
			}
			
			if (!empty($product->meta_keywords)){
            $this->design->assign('meta_keywords', $product->meta_keywords);
			}elseif(!empty($category->auto_meta_keywords)){
				$this->design->assign('meta_keywords', $auto_meta_keywords);
			}else{ $this->design->assign('meta_keywords', mb_strtolower($all_p_meta_keywords));
			}
			
			if (!empty($product->meta_description)){
            $this->design->assign('meta_description', $product->meta_description);
			}elseif(!empty($category->auto_meta_desc)){
				$this->design->assign('meta_description', $auto_meta_description);
			}else{$this->design->assign('meta_description', $all_p_meta_description);
			}
		}
		
        /*поставщики*/
        if(isset($_SESSION['admin'])){
            $product_suppliers = $this->suppliers->get_suppliers(array('product_id'=>$product->id));
            $variant_for_suppliers = array();
            $vfs_ids = array();
            foreach($this->variants->get_variants(array('product_id'=>$product->id)) as $vfs){
                $variant_for_suppliers[$vfs->id] = $vfs;
                $vfs_ids[] = $vfs->id;
            }
            foreach($product_suppliers as &$ps){
                foreach($vfs_ids as $vid)
                    if($values = $this->suppliers->get_variant_supplier_info(array('variant_id'=>$vid,'supplier_id'=>$ps->id))){
                        $ps->variants[$vid] = reset($values);
                        $ps->variants[$vid]->bonus = $variant_for_suppliers[$vid]->price - $ps->variants[$vid]->price;
                    }
            }
            $this->design->assign('variant_for_suppliers',$variant_for_suppliers);
            $this->design->assign('product_suppliers', $product_suppliers);
            $this->design->assign('product_supplier', reset($product_suppliers));
        }
        /*/поставщики*/
        return $this->design->fetch('product.tpl');
    }
    
}
